@@ -96,6 +96,9 @@ released_cb (GtkGestureClick *gesture,
   if (self->swipe_active)
     return;
 
+  if (!self->can_close)
+    return;
+
   adw_bottom_sheet_set_open (self, FALSE);
 }
 
@@ -120,6 +123,12 @@ open_animation_done_cb (AdwBottomSheet *self)
   }
 }
 
+static void
+sheet_close_cb (AdwBottomSheet *self)
+{
+  adw_bottom_sheet_set_open (self, FALSE);
+}
+
 static void
 measure_sheet (GtkWidget      *widget,
                GtkOrientation  orientation,
@@ -201,14 +210,6 @@ allocate_sheet (GtkWidget *widget,
     gtk_widget_allocate (self->outline, width, height, baseline, NULL);
 }
 
-static gboolean
-adw_bottom_sheet_contains (GtkWidget *widget,
-                           double     x,
-                           double     y)
-{
-  return FALSE;
-}
-
 static void
 adw_bottom_sheet_measure (GtkWidget      *widget,
                           GtkOrientation  orientation,
@@ -397,11 +398,13 @@ adw_bottom_sheet_class_init (AdwBottomSheetClass *klass)
   object_class->get_property = adw_bottom_sheet_get_property;
   object_class->set_property = adw_bottom_sheet_set_property;
 
-  widget_class->contains = adw_bottom_sheet_contains;
+  widget_class->contains = adw_widget_contains_passthrough;
   widget_class->measure = adw_bottom_sheet_measure;
   widget_class->size_allocate = adw_bottom_sheet_size_allocate;
   widget_class->get_request_mode = adw_widget_get_request_mode;
   widget_class->compute_expand = adw_widget_compute_expand;
+  widget_class->focus = adw_widget_focus_child;
+  widget_class->grab_focus = adw_widget_grab_focus_child;
 
   props[PROP_CHILD] =
     g_param_spec_object ("child", NULL, NULL,
@@ -466,6 +469,9 @@ adw_bottom_sheet_class_init (AdwBottomSheetClass *klass)
                               G_TYPE_FROM_CLASS (klass),
                               adw_marshal_VOID__VOIDv);
 
+  gtk_widget_class_install_action (widget_class, "sheet.close", NULL,
+                                   (GtkWidgetActionActivateFunc) sheet_close_cb);
+
   gtk_widget_class_set_css_name (widget_class, "bottom-sheet");
 }
 
@@ -556,7 +562,8 @@ adw_bottom_sheet_init (AdwBottomSheet *self)
 
   self->sheet_bin = adw_gizmo_new ("sheet", NULL, NULL, NULL, NULL,
                                    (AdwGizmoFocusFunc) adw_widget_focus_child,
-                                   (AdwGizmoGrabFocusFunc) adw_widget_grab_focus_child);
+                                   (AdwGizmoGrabFocusFunc) adw_widget_grab_focus_child_or_self);
+  gtk_widget_set_focusable (self->sheet_bin, TRUE);
   gtk_widget_set_layout_manager (self->sheet_bin,
                                  gtk_custom_layout_new (adw_widget_get_request_mode,
                                                         measure_sheet,
@@ -569,7 +576,9 @@ adw_bottom_sheet_init (AdwBottomSheet *self)
   self->drag_handle = adw_gizmo_new ("drag-handle", NULL, NULL, NULL, NULL, NULL, NULL);
   gtk_widget_set_parent (self->drag_handle, self->sheet_bin);
 
-  self->outline = adw_gizmo_new ("outline", NULL, NULL, NULL, NULL, NULL, NULL);
+  self->outline = adw_gizmo_new ("outline", NULL, NULL, NULL,
+                                 (AdwGizmoContainsFunc) adw_widget_contains_passthrough,
+                                 NULL, NULL);
   gtk_widget_set_can_target (self->outline, FALSE);
   gtk_widget_set_can_focus (self->outline, FALSE);
   gtk_widget_set_parent (self->outline, self->sheet_bin);
@@ -960,22 +969,6 @@ adw_bottom_sheet_set_can_close (AdwBottomSheet *self,
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CAN_CLOSE]);
 }
 
-void
-adw_bottom_sheet_set_dialog_mode (AdwBottomSheet *self,
-                                  gboolean        dialog_mode)
-{
-  GtkAccessibleRole role;
-
-  g_return_if_fail (ADW_IS_BOTTOM_SHEET (self));
-
-  if (dialog_mode)
-    role = GTK_ACCESSIBLE_ROLE_DIALOG;
-  else
-    role = GTK_ACCESSIBLE_ROLE_GENERIC;
-
-  g_object_set (self->sheet_bin, "accessible-role", role, NULL);
-}
-
 void
 adw_bottom_sheet_set_min_natural_width (AdwBottomSheet *self,
                                         int             min_natural_width)
@@ -984,3 +977,11 @@ adw_bottom_sheet_set_min_natural_width (AdwBottomSheet *self,
 
   self->min_natural_width = min_natural_width;
 }
+
+GtkWidget *
+adw_bottom_sheet_get_sheet_bin (AdwBottomSheet *self)
+{
+  g_return_val_if_fail (ADW_IS_BOTTOM_SHEET (self), NULL);
+
+  return self->sheet_bin;
+}
