@@ -14,12 +14,6 @@
 #include "adw-dialog-private.h"
 #include "adw-widget-utils-private.h"
 
-#ifdef GDK_WINDOWING_MACOS
-#import <AppKit/AppKit.h>
-
-#include <gdk/macos/gdkmacos.h>
-#endif
-
 struct _AdwDialogHost
 {
   GtkWidget parent_instance;
@@ -35,12 +29,6 @@ struct _AdwDialogHost
   GtkWidget *last_focus;
 
   GtkWidget *proxy;
-
-#ifdef GDK_WINDOWING_MACOS
-  guint close_enabled: 1;
-  guint minimize_enabled: 1;
-  guint maximize_enabled: 1;
-#endif
 };
 
 static void adw_dialog_host_buildable_init (GtkBuildableIface *iface);
@@ -196,16 +184,6 @@ dialog_closing_cb (AdwDialog     *dialog,
     adw_dialog_set_shadowed (next_dialog, FALSE);
   }
 
-  #ifdef GDK_WINDOWING_MACOS
-  if (self->dialogs->len == 0 && GDK_IS_MACOS_DISPLAY (gdk_display_get_default ())) {
-    NSWindow *nswindow = (NSWindow*) gdk_macos_surface_get_native_window (GDK_MACOS_SURFACE (gtk_native_get_surface (GTK_NATIVE (root))));
-
-    [[nswindow standardWindowButton:NSWindowCloseButton] setEnabled:self->close_enabled];
-    [[nswindow standardWindowButton:NSWindowMiniaturizeButton] setEnabled:self->minimize_enabled];
-    [[nswindow standardWindowButton:NSWindowZoomButton] setEnabled:self->maximize_enabled];
-  }
-#endif
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_VISIBLE_DIALOG]);
 }
 
@@ -226,6 +204,15 @@ dialog_remove_cb (AdwDialog     *dialog,
     gtk_widget_unparent (GTK_WIDGET (dialog));
 }
 
+static gboolean
+key_pressed_cb (AdwDialogHost *self)
+{
+  if (self->dialogs->len == 0)
+    return GDK_EVENT_PROPAGATE;
+
+  return GDK_EVENT_STOP;
+}
+
 static void
 adw_dialog_host_root (GtkWidget *widget)
 {
@@ -273,14 +260,6 @@ adw_dialog_host_unmap (GtkWidget *widget)
                             self->dialogs_closed_during_unmap->len);
 }
 
-static GtkSizeRequestMode
-adw_dialog_host_get_request_mode (GtkWidget *widget)
-{
-  AdwDialogHost *self = ADW_DIALOG_HOST (widget);
-
-  return gtk_widget_get_request_mode (self->bin);
-}
-
 static void
 adw_dialog_host_measure (GtkWidget      *widget,
                          GtkOrientation  orientation,
@@ -292,10 +271,6 @@ adw_dialog_host_measure (GtkWidget      *widget,
 {
   AdwDialogHost *self = ADW_DIALOG_HOST (widget);
 
-  /* Only measure the child, not any dialogs. In case a dialog is too
-   * large to fit the screen (e.g. on a phone), we'd rather clip the
-   * dialog than have the whole window request a large size and overflow.
-   */
   gtk_widget_measure (self->bin, orientation, for_size,
                       minimum, natural, minimum_baseline, natural_baseline);
 }
@@ -311,10 +286,14 @@ adw_dialog_host_size_allocate (GtkWidget *widget,
   for (child = gtk_widget_get_first_child (widget);
        child;
        child = gtk_widget_get_next_sibling (child)) {
-    GtkAllocation child_allocation = { 0, 0, width, height };
+    GtkRequisition min;
 
-    adw_ensure_child_allocation_size (child, &child_allocation);
-    gtk_widget_size_allocate (child, &child_allocation, -1);
+    gtk_widget_get_preferred_size (child, &min, NULL);
+
+    width = MAX (width, min.width);
+    height = MAX (height, min.height);
+
+    gtk_widget_allocate (child, width, height, baseline, NULL);
   }
 }
 
@@ -414,7 +393,7 @@ adw_dialog_host_class_init (AdwDialogHostClass *klass)
   widget_class->unmap = adw_dialog_host_unmap;
   widget_class->measure = adw_dialog_host_measure;
   widget_class->size_allocate = adw_dialog_host_size_allocate;
-  widget_class->get_request_mode = adw_dialog_host_get_request_mode;
+  widget_class->get_request_mode = adw_widget_get_request_mode;
   widget_class->compute_expand = adw_widget_compute_expand;
 
   props[PROP_CHILD] =
@@ -440,6 +419,8 @@ adw_dialog_host_class_init (AdwDialogHostClass *klass)
 static void
 adw_dialog_host_init (AdwDialogHost *self)
 {
+  GtkEventController *controller;
+
   self->dialogs = g_ptr_array_new ();
 
   self->dialogs_closed_during_unmap = g_ptr_array_new ();
@@ -447,11 +428,9 @@ adw_dialog_host_init (AdwDialogHost *self)
   self->bin = adw_bin_new ();
   gtk_widget_set_parent (self->bin, GTK_WIDGET (self));
 
-#ifdef GDK_WINDOWING_MACOS
-  self->close_enabled = TRUE;
-  self->minimize_enabled = TRUE;
-  self->maximize_enabled = TRUE;
-#endif
+  controller = gtk_event_controller_key_new ();
+  g_signal_connect_swapped (controller, "key-pressed", G_CALLBACK (key_pressed_cb), self);
+  gtk_widget_add_controller (GTK_WIDGET (self), controller);
 }
 
 static void
@@ -495,12 +474,12 @@ adw_dialog_host_set_child (AdwDialogHost *self,
   g_return_if_fail (ADW_IS_DIALOG_HOST (self));
   g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
 
-  if (adw_dialog_host_get_child (self) == child)
-    return;
-
   if (child)
     g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
+  if (adw_dialog_host_get_child (self) == child)
+    return;
+
   adw_bin_set_child (ADW_BIN (self->bin), child);
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_CHILD]);
@@ -590,20 +569,6 @@ adw_dialog_host_present_dialog (AdwDialogHost *self,
     gtk_widget_insert_before (GTK_WIDGET (dialog), GTK_WIDGET (self), NULL);
   }
 
-#ifdef GDK_WINDOWING_MACOS
-  if (self->dialogs->len == 0 && GDK_IS_MACOS_DISPLAY (gdk_display_get_default ())) {
-    NSWindow *nswindow = (NSWindow*) gdk_macos_surface_get_native_window (GDK_MACOS_SURFACE (gtk_native_get_surface (GTK_NATIVE (root))));
-
-    self->close_enabled = [[nswindow standardWindowButton:NSWindowCloseButton] isEnabled];
-    self->minimize_enabled = [[nswindow standardWindowButton:NSWindowMiniaturizeButton] isEnabled];
-    self->maximize_enabled = [[nswindow standardWindowButton:NSWindowZoomButton] isEnabled];
-
-    [[nswindow standardWindowButton:NSWindowCloseButton] setEnabled:NO];
-    [[nswindow standardWindowButton:NSWindowMiniaturizeButton] setEnabled:NO];
-    [[nswindow standardWindowButton:NSWindowZoomButton] setEnabled:NO];
-  }
-#endif
-
   g_ptr_array_add (self->dialogs, dialog);
 
   if (self->dialogs_model)
