@@ -81,7 +81,7 @@
  * ## Header Bar Integration
  *
  * When placed inside an `AdwDialog`, [class@HeaderBar] will display the dialog
- * title intead of window title. It will also adjust the decoration layout to
+ * title instead of window title. It will also adjust the decoration layout to
  * ensure it always has a close button and nothing else. Set
  * [property@HeaderBar:show-start-title-buttons] and
  * [property@HeaderBar:show-end-title-buttons] to `FALSE` to remove it if it's
@@ -314,11 +314,18 @@ window_notify_focus_cb (AdwDialog  *self,
                         GParamSpec *pspec,
                         GtkRoot    *root)
 {
+  AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
   GtkWidget *focus = gtk_root_get_focus (root);
 
   if (focus && !gtk_widget_is_ancestor (focus, GTK_WIDGET (self)))
     focus = NULL;
 
+  if (priv->floating_sheet && focus == adw_floating_sheet_get_sheet_bin (priv->floating_sheet))
+    focus = NULL;
+
+  if (priv->bottom_sheet && focus == adw_bottom_sheet_get_sheet_bin (priv->bottom_sheet))
+    focus = NULL;
+
   set_focus (self, focus);
 }
 
@@ -451,7 +458,6 @@ update_presentation (AdwDialog *self)
   if (use_bottom_sheet) {
     priv->bottom_sheet = ADW_BOTTOM_SHEET (adw_bottom_sheet_new ());
 
-    adw_bottom_sheet_set_dialog_mode (priv->bottom_sheet, TRUE);
     adw_bottom_sheet_set_min_natural_width (priv->bottom_sheet, 360);
 
     if (!priv->first_map)
@@ -566,11 +572,18 @@ window_close_request_cb (AdwDialog *self)
 {
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
 
-  if (priv->force_closing)
-    return GDK_EVENT_PROPAGATE;
+  if (priv->force_closing || adw_dialog_close (self)) {
+    GtkAccessibleRole role =
+      gtk_accessible_get_accessible_role (GTK_ACCESSIBLE (priv->window));
+
+    g_object_set (self, "accessible-role", role, NULL);
+
+    gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                    GTK_ACCESSIBLE_PROPERTY_LABEL, priv->title,
+                                    -1);
 
-  if (adw_dialog_close (self))
     return GDK_EVENT_PROPAGATE;
+  }
 
   return GDK_EVENT_STOP;
 }
@@ -581,6 +594,7 @@ present_as_window (AdwDialog *self,
 {
   AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
   GtkWidget *titlebar;
+  GtkAccessibleRole role;
 
   if (priv->window) {
     gtk_window_present (GTK_WINDOW (priv->window));
@@ -588,6 +602,7 @@ present_as_window (AdwDialog *self,
   }
 
   gtk_widget_add_css_class (GTK_WIDGET (self), "floating");
+  gtk_widget_set_focusable (GTK_WIDGET (self), TRUE);
 
   priv->window = gtk_window_new ();
   gtk_window_set_resizable (GTK_WINDOW (priv->window), FALSE);
@@ -616,6 +631,13 @@ present_as_window (AdwDialog *self,
 
   g_signal_connect_swapped (priv->window, "close-request", G_CALLBACK (window_close_request_cb), self);
 
+  /* Transfer the accessible role onto the window */
+  role = gtk_accessible_get_accessible_role (GTK_ACCESSIBLE (self));
+  g_object_set (priv->window, "accessible-role", role, NULL);
+  g_object_set (self, "accessible-role", GTK_ACCESSIBLE_ROLE_GENERIC, NULL);
+
+  gtk_accessible_reset_property (GTK_ACCESSIBLE (self), GTK_ACCESSIBLE_PROPERTY_LABEL);
+
   gtk_window_present (GTK_WINDOW (priv->window));
 }
 
@@ -715,6 +737,34 @@ open_inspector_cb (AdwDialog *self,
   return ret;
 }
 
+static gboolean
+ensure_focus (AdwDialog *self)
+{
+  AdwDialogPrivate *priv = adw_dialog_get_instance_private (self);
+  GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
+  GtkWidget *focus;
+
+  if (!root)
+    return FALSE;
+
+  focus = gtk_root_get_focus (root);
+
+  if (focus)
+    return FALSE;
+
+  /* No focusable widgets, focus something intermediate instead */
+  if (priv->floating_sheet)
+    return gtk_widget_grab_focus (GTK_WIDGET (priv->floating_sheet));
+
+  if (priv->bottom_sheet)
+    return gtk_widget_grab_focus (GTK_WIDGET (priv->bottom_sheet));
+
+  if (priv->window)
+    return adw_widget_grab_focus_self (GTK_WIDGET (self));
+
+  return TRUE;
+}
+
 static void
 adw_dialog_root (GtkWidget *widget)
 {
@@ -771,6 +821,16 @@ adw_dialog_map (GtkWidget *widget)
                                                    NULL, NULL);
 }
 
+static gboolean
+adw_dialog_focus (GtkWidget        *widget,
+                  GtkDirectionType  direction)
+{
+  if (adw_widget_focus_child (widget, direction))
+    return TRUE;
+
+  return ensure_focus (ADW_DIALOG (widget));
+}
+
 static gboolean
 adw_dialog_grab_focus (GtkWidget *widget)
 {
@@ -782,15 +842,8 @@ adw_dialog_grab_focus (GtkWidget *widget)
 
   GTK_WIDGET_GET_CLASS (widget)->move_focus (GTK_WIDGET (widget),
                                              GTK_DIR_TAB_FORWARD);
-  return TRUE;
-}
 
-static gboolean
-adw_dialog_contains (GtkWidget *widget,
-                     double     x,
-                     double     y)
-{
-  return FALSE;
+  return ensure_focus (ADW_DIALOG (widget));
 }
 
 static void
@@ -952,9 +1005,9 @@ adw_dialog_class_init (AdwDialogClass *klass)
   widget_class->root = adw_dialog_root;
   widget_class->unroot = adw_dialog_unroot;
   widget_class->map = adw_dialog_map;
-  widget_class->focus = adw_widget_focus_child;
+  widget_class->focus = adw_dialog_focus;
   widget_class->grab_focus = adw_dialog_grab_focus;
-  widget_class->contains = adw_dialog_contains;
+  widget_class->contains = adw_widget_contains_passthrough;
   widget_class->compute_expand = adw_widget_compute_expand;
 
   /**
@@ -1190,6 +1243,7 @@ adw_dialog_class_init (AdwDialogClass *klass)
 
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, "dialog");
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_DIALOG);
 }
 
 static void
@@ -1354,6 +1408,10 @@ adw_dialog_set_title (AdwDialog  *self,
     return;
 
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE]);
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+                                  GTK_ACCESSIBLE_PROPERTY_LABEL, priv->title,
+                                  -1);
 }
 
 /**
@@ -2047,19 +2105,22 @@ adw_dialog_set_shadowed (AdwDialog *self,
 
   shadowed = !!shadowed;
 
-  if (shadowed)
-    g_set_weak_pointer (&priv->last_focus, priv->focus_widget);
+  if (shadowed) {
+    GtkWidget *focus = priv->focus_widget;
+
+    while (focus && !gtk_widget_get_mapped (focus))
+      focus = gtk_widget_get_parent (focus);
+
+    if (focus && gtk_widget_is_ancestor (focus, priv->child_breakpoint_bin))
+      g_set_weak_pointer (&priv->last_focus, focus);
+  }
 
   gtk_widget_set_can_focus (priv->bin, !shadowed);
   gtk_widget_set_can_target (priv->bin, !shadowed);
 
   if (!shadowed) {
-    if (priv->last_focus) {
-      GtkRoot *root = gtk_widget_get_root (GTK_WIDGET (self));
-
-      if (root)
-        gtk_root_set_focus (root, priv->last_focus);
-    }
+    if (priv->last_focus)
+      gtk_widget_grab_focus (priv->last_focus);
 
     g_clear_weak_pointer (&priv->last_focus);
   }
